
#include "CGInterface.h"
#include "v3.h"
#include "scene.h"

#include <iostream>

using namespace std;

CGInterface::CGInterface() : needInit(true) {};

void CGInterface::PerSessionInit() {

  glEnable(GL_DEPTH_TEST);

  CGprofile latestVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
  CGprofile latestGeometryProfile = cgGLGetLatestProfile(CG_GL_GEOMETRY);
  CGprofile latestPixelProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);

  if (latestGeometryProfile == CG_PROFILE_UNKNOWN) {
    cerr << "ERROR: geometry profile is not available" << endl;
    exit(0);
  }

  cgGLSetOptimalOptions(latestGeometryProfile);
  CGerror Error = cgGetError();
  if (Error) {
	  cerr << "CG ERROR: " << cgGetErrorString(Error) << endl;
  }

  cout << "Info: Latest GP Profile Supported: " << cgGetProfileString(latestGeometryProfile) << endl;

  geometryCGprofile = latestGeometryProfile;

  cout << "Info: Latest VP Profile Supported: " << cgGetProfileString(latestVertexProfile) << endl;
  cout << "Info: Latest FP Profile Supported: " << cgGetProfileString(latestPixelProfile) << endl;

  vertexCGprofile = latestVertexProfile;
  pixelCGprofile = latestPixelProfile;
  cgContext = cgCreateContext();  

  needInit = false;

}

bool ShaderOneInterface::PerSessionInit(CGInterface *cgi) {

  geometryProgram = cgCreateProgramFromFile(cgi->cgContext, CG_SOURCE, 
    "CG/shaderOne.cg", cgi->geometryCGprofile, "GeometryMain", NULL);
  if (geometryProgram == NULL)  {
    CGerror Error = cgGetError();
    cerr << "Shader One Geometry Program COMPILE ERROR: " << cgGetErrorString(Error) << endl;
    cerr << cgGetLastListing(cgi->cgContext) << endl << endl;
    return false;
  }

  vertexProgram = cgCreateProgramFromFile(cgi->cgContext, CG_SOURCE, 
    "CG/shaderOne.cg", cgi->vertexCGprofile, "VertexMain", NULL);
  if (vertexProgram == NULL) {
    CGerror Error = cgGetError();
    cerr << "Shader One Geometry Program COMPILE ERROR: " << cgGetErrorString(Error) << endl;
    cerr << cgGetLastListing(cgi->cgContext) << endl << endl;
    return false;
  }

  fragmentProgram = cgCreateProgramFromFile(cgi->cgContext, CG_SOURCE, 
    "CG/shaderOne.cg", cgi->pixelCGprofile, "FragmentMain", NULL);
  if (fragmentProgram == NULL)  {
    CGerror Error = cgGetError();
    cerr << "Shader One Fragment Program COMPILE ERROR: " << cgGetErrorString(Error) << endl;
    cerr << cgGetLastListing(cgi->cgContext) << endl << endl;
    return false;
  }

	// load programs
	cgGLLoadProgram(geometryProgram);
	cgGLLoadProgram(vertexProgram);
	cgGLLoadProgram(fragmentProgram);

	// build some parameters by name such that we can set them later...
  vertexModelViewProj = cgGetNamedParameter(vertexProgram, "modelViewProj" );
  geometryModelViewProj = cgGetNamedParameter(geometryProgram, "modelViewProj" );
  geometrySF = cgGetNamedParameter(geometryProgram, "sf" );
  fragmentBlueHue = cgGetNamedParameter(fragmentProgram, "blueHue" );
  fragmentLightDirv = cgGetNamedParameter(fragmentProgram, 
    "lightDirv" );

  return true;

}

void ShaderOneInterface::PerFrameInit() {

	//set parameters
	cgGLSetStateMatrixParameter(vertexModelViewProj, 
		CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);

  cgGLSetStateMatrixParameter(geometryModelViewProj, 
		CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);

  static float blueHue = 0.5f;
  cgGLSetParameter1f(fragmentBlueHue, blueHue);
  cgGLSetParameter3fv(fragmentLightDirv, (float*)&(scene->lightDirv));
  cgGLSetParameter1f(geometrySF, scene->sf);

}

void ShaderOneInterface::PerFrameDisable() {
}


void ShaderOneInterface::BindPrograms() {

  cgGLBindProgram( geometryProgram);
  cgGLBindProgram( vertexProgram);
  cgGLBindProgram( fragmentProgram);

}

void CGInterface::DisableProfiles() {

  cgGLDisableProfile(vertexCGprofile);
  cgGLDisableProfile(geometryCGprofile);
  cgGLDisableProfile(pixelCGprofile);

}

void CGInterface::EnableProfiles() {

  cgGLEnableProfile(vertexCGprofile);
  cgGLEnableProfile(geometryCGprofile);
  cgGLEnableProfile(pixelCGprofile);

}

#if 0

for tmi = 0 to tmN
  for vi = 0 to tm[tmi].vertsN
    if fixedPipeline
      Project(tm[tmi].verts[vi]) // fixed pipeline
    else

      VertexShader(tm[tmi].verts[vi]) // programmable pipeline

  for ti = 0 to tm[tmi].trisN
    aabb = ComputeAABB(tm[tmi], ti)
    caabb = Clip(AABB, frame)
    if (caabb empty)
      continue
    eeqs = SetupEdgeEquations(tm[tmi], ti)
    SSISetup(tm[tmi], ti, rastParameters)
    MSISetup(tm[tmi], ti, rastParameters)

    if programmablePipeline
      GeometryShader(tm[tmi], ti) // programmable pipeline

    for v = caabb.top to caabb.bottom
      for u = caabb.left to caabb.right
        if Outside(u, v, eeqs)
          continue
        currz = ABCz * (u, v, 1)
        if Hidden(u, v, currz, fb->zb)
          continue

          // rasterization parameter interpolation
          currs = (ABCs * (u, v, 1)) / (DEF * (u, v, 1))
          currt = (ABCt * (u, v, 1)) / (DEF * (u, v, 1))
          currColor.red = (ABCred * (u, v, 1)) / (DEF * (u, v, 1))
          currColor.green = (ABCred * (u, v, 1)) / (DEF * (u, v, 1))
          currColor.blue = (ABCred * (u, v, 1)) / (DEF * (u, v, 1))
          //...

        if fixedPipeline
            if tm
              color(u, v) = TextureMapping(currs, currt)
            if vertexColorInterpolation
              color(u, v) = currColor
            if lighting
              color(u, v) = Light()
            if shadows
              color(u, v) = ShadowMap(u, v)

        else
            color = PixelShader(u, v, currs, currt, currColor, ...) // programmable pipeline

#endif